@@ -21,7 +21,7 @@ module IRB # :nodoc:
     IRB.load_modules
 
     unless @CONF[:PROMPT][@CONF[:PROMPT_MODE]]
-      IRB.fail(UndefinedPromptMode, @CONF[:PROMPT_MODE])
+      fail UndefinedPromptMode, @CONF[:PROMPT_MODE]
     end
   end
 
@@ -217,7 +217,7 @@ module IRB # :nodoc:
         end
         break
       when /^-/
-        IRB.fail UnrecognizedSwitch, opt
+        fail UnrecognizedSwitch, opt
       else
         @CONF[:SCRIPT] = opt
         $0 = opt
@@ -262,7 +262,7 @@ module IRB # :nodoc:
     when String
       return rc_file
     else
-      IRB.fail IllegalRCNameGenerator
+      fail IllegalRCNameGenerator
     end
   end
 
@@ -271,10 +271,19 @@ module IRB # :nodoc:
     if irbrc = ENV["IRBRC"]
       yield proc{|rc| rc == "rc" ? irbrc : irbrc+rc}
     end
+    if xdg_config_home = ENV["XDG_CONFIG_HOME"]
+      irb_home = File.join(xdg_config_home, "irb")
+      unless File.exist? irb_home
+        require 'fileutils'
+        FileUtils.mkdir_p irb_home
+      end
+      yield proc{|rc| irb_home + "/irb#{rc}"}
+    end
     if home = ENV["HOME"]
       yield proc{|rc| home+"/.irb#{rc}"}
     end
     current_dir = Dir.pwd
+    yield proc{|rc| current_dir+"/.config/irb/irb#{rc}"}
     yield proc{|rc| current_dir+"/.irb#{rc}"}
     yield proc{|rc| current_dir+"/irb#{rc.sub(/\A_?/, '.')}"}
     yield proc{|rc| current_dir+"/_irb#{rc}"}
@@ -296,15 +305,18 @@ module IRB # :nodoc:
   DefaultEncodings = Struct.new(:external, :internal)
   class << IRB
     private
-    def set_encoding(extern, intern = nil)
+    def set_encoding(extern, intern = nil, override: true)
       verbose, $VERBOSE = $VERBOSE, nil
       Encoding.default_external = extern unless extern.nil? || extern.empty?
       Encoding.default_internal = intern unless intern.nil? || intern.empty?
-      @CONF[:ENCODINGS] = IRB::DefaultEncodings.new(extern, intern)
       [$stdin, $stdout, $stderr].each do |io|
         io.set_encoding(extern, intern)
       end
-      @CONF[:LC_MESSAGES].instance_variable_set(:@encoding, extern)
+      if override
+        @CONF[:LC_MESSAGES].instance_variable_set(:@override_encoding, extern)
+      else
+        @CONF[:LC_MESSAGES].instance_variable_set(:@encoding, extern)
+      end
     ensure
       $VERBOSE = verbose
     end
