@@ -43,6 +43,11 @@ module IRB
     def readable_after_eof?
       false
     end
+
+    # For debug message
+    def inspect
+      'Abstract InputMethod'
+    end
   end
 
   class StdioInputMethod < InputMethod
@@ -93,6 +98,11 @@ module IRB
     def encoding
       @stdin.external_encoding
     end
+
+    # For debug message
+    def inspect
+      'StdioInputMethod'
+    end
   end
 
   # Use a File for IO with irb, see InputMethod
@@ -125,6 +135,11 @@ module IRB
     def encoding
       @io.external_encoding
     end
+
+    # For debug message
+    def inspect
+      'FileInputMethod'
+    end
   end
 
   begin
@@ -133,6 +148,9 @@ module IRB
       include Readline
       # Creates a new input method object using Readline
       def initialize
+        if Readline.respond_to?(:encoding_system_needs)
+          IRB.__send__(:set_encoding, Readline.encoding_system_needs.name, override: false)
+        end
         super
 
         @line_no = 0
@@ -199,6 +217,15 @@ module IRB
       end
       Readline.completion_append_character = nil
       Readline.completion_proc = IRB::InputCompletor::CompletionProc
+
+      # For debug message
+      def inspect
+        readline_impl = (defined?(Reline) && Readline == Reline) ? 'Reline' : 'ext/readline'
+        str = "ReadlineInputMethod with #{readline_impl} #{Readline::VERSION}"
+        inputrc_path = File.expand_path(ENV['INPUTRC'] || '~/.inputrc')
+        str += " and #{inputrc_path}" if File.exist?(inputrc_path)
+        str
+      end
     end
   rescue LoadError
   end
@@ -207,6 +234,7 @@ module IRB
     include Reline
     # Creates a new input method object using Readline
     def initialize
+      IRB.__send__(:set_encoding, Reline.encoding_system_needs.name, override: false)
       super
 
       @line_no = 0
@@ -293,5 +321,18 @@ module IRB
     def encoding
       @stdin.external_encoding
     end
+
+    # For debug message
+    def inspect
+      config = Reline::Config.new
+      str = "ReidlineInputMethod with Reline #{Reline::VERSION}"
+      if config.respond_to?(:inputrc_path)
+        inputrc_path = config.inputrc_path
+      else
+        inputrc_path = File.expand_path(ENV['INPUTRC'] || '~/.inputrc')
+      end
+      str += " and #{inputrc_path}" if File.exist?(inputrc_path)
+      str
+    end
   end
 end
